@@ -3,6 +3,7 @@ from operator import itemgetter, methodcaller
 from collections import Mapping, Set, Iterable, Iterator, defaultdict
 from itertools import chain, tee
 
+from .primitives import EMPTY
 from .funcs import identity, partial, complement
 from .funcmakers import wrap_mapper, wrap_selector
 from .seqs import take, imap, ifilter
@@ -12,13 +13,10 @@ __all__ = ['empty', 'iteritems',
            'join', 'merge',
            'walk', 'walk_keys', 'walk_values', 'select', 'select_keys', 'select_values', 'compact',
            'is_distinct', 'all', 'any', 'none', 'one', 'some',
-           'zipdict', 'flip', 'project',
+           'zipdict', 'flip', 'project', 'izip_values', 'izip_dicts',
            'where', 'pluck', 'invoke']
 
 
-EMPTY = object()
-
-
 ### Generic ops
 def _factory(coll):
     # Hack for defaultdicts overriden constructor
@@ -34,10 +32,6 @@ def _factory(coll):
 def empty(coll):
     return _factory(coll)()
 
-# TODO: not_empty/non_empty working with iterators?
-#       kinda bool(iterator)
-# NOTE: should I add it to seqs or even data?
-
 def iteritems(coll):
     return coll.iteritems() if hasattr(coll, 'iteritems') else coll
 
@@ -146,8 +140,6 @@ def some(pred, seq=EMPTY):
 # def chain_test(compare, seq):
 #     return all(compare, izip(seq, rest(seq))
 
-# TODO: capabilities + type tests or skip?
-
 def zipdict(keys, vals):
     return dict(zip(keys, vals))
 
@@ -157,6 +149,20 @@ def flip(mapping):
 def project(mapping, keys):
     return _factory(mapping)((k, mapping[k]) for k in keys if k in mapping)
 
+def izip_values(*dicts):
+    if len(dicts) < 1:
+        raise TypeError('izip_values expects at least one argument')
+    keys = set.intersection(*map(set, dicts))
+    for key in keys:
+        yield tuple(d[key] for d in dicts)
+
+def izip_dicts(*dicts):
+    if len(dicts) < 1:
+        raise TypeError('izip_dicts expects at least one argument')
+    keys = set.intersection(*map(set, dicts))
+    for key in keys:
+        yield tuple([key] + [d[key] for d in dicts])
+
 
 def where(mappings, **cond):
     match = lambda m: all(m[k] == v for k, v in cond.items())
