@@ -4,20 +4,21 @@ from itertools import islice, ifilter, imap, izip, chain, tee, ifilterfalse, dro
 from collections import defaultdict, deque, Sequence
 
 from .primitives import EMPTY
+from .types import is_seqcont
 from .funcs import partial
 from .funcmakers import wrap_mapper, wrap_selector
 
 
 __all__ = [
     'count', 'cycle', 'repeat', 'repeatedly', 'iterate',
-    'take', 'drop', 'first', 'second', 'last', 'rest', 'butlast', 'ilen',
+    'take', 'drop', 'first', 'second', 'nth', 'last', 'rest', 'butlast', 'ilen',
     'map', 'filter', 'imap', 'ifilter', 'remove', 'iremove', 'keep', 'ikeep', 'without', 'iwithout',
-    'concat', 'iconcat', 'chain', 'cat', 'icat', 'mapcat', 'imapcat',
+    'concat', 'iconcat', 'chain', 'cat', 'icat', 'flatten', 'iflatten', 'mapcat', 'imapcat',
     'izip', 'interleave', 'interpose', 'distinct',
     'dropwhile', 'takewhile', 'split', 'split_at', 'split_by',
     'group_by', 'count_by',
     'partition', 'ipartition', 'chunks', 'ichunks', 'ipartition_by', 'partition_by',
-    'with_prev',
+    'with_prev', 'pairwise',
     'ireductions', 'reductions', 'isums', 'sums',
 ]
 
@@ -47,6 +48,11 @@ def first(seq):
 def second(seq):
     return first(rest(seq))
 
+# TODO: decide how negative indexes should work - count from end or just return None?
+#       it raises ValueError for now
+def nth(n, seq):
+    return next(islice(seq, n, n+1), None)
+
 def last(seq):
     if hasattr(seq, '__getslice__'):
         try:
@@ -108,7 +114,7 @@ def ikeep(f, seq=EMPTY):
 
 def iwithout(seq, *items):
     for value in seq:
-        if not value in items:
+        if value not in items:
             yield value
 
 def without(seq, *items):
@@ -123,6 +129,17 @@ def cat(seqs):
     return list(icat(seqs))
 icat = chain.from_iterable
 
+def iflatten(seq, follow=is_seqcont):
+    for item in seq:
+        if follow(item):
+            for sub in iflatten(item, is_seqcont):
+                yield sub
+        else:
+            yield item
+
+def flatten(seq, follow=is_seqcont):
+    return list(iflatten(seq, follow))
+
 def mapcat(f, *seqs):
     return cat(imap(f, *seqs))
 
@@ -205,7 +222,7 @@ def _icut_iter(drop_tail, n, step, seq):
 def _icut(drop_tail, n, step, seq=EMPTY):
     if seq is EMPTY:
         step, seq = n, step
-    if isinstance(seq, Sequence):
+    if isinstance(seq, Sequence) and not isinstance(seq, xrange):
         return _icut_seq(drop_tail, n, step, seq)
     else:
         return _icut_iter(drop_tail, n, step, seq)
@@ -235,6 +252,13 @@ def with_prev(seq, fill=None):
     a, b = tee(seq)
     return izip(a, chain([fill], b))
 
+# An itertools recipe
+# NOTE: this is the same as ipartition(2, 1, seq) only faster and with distinct name
+def pairwise(seq):
+    a, b = tee(seq)
+    next(b, None)
+    return izip(a, b)
+
 
 def ireductions(f, seq, acc=EMPTY):
     it = iter(seq)
